from contextlib import contextmanager
from enum import IntEnum
import os 

class CERTIFIER_OPT(IntEnum):
    CERTIFIER_OPT_CFG_FILENAME         = 1
    CERTIFIER_OPT_AUTH_TYPE            = 2
    CERTIFIER_OPT_CERTIFIER_URL        = 3
    CERTIFIER_OPT_HTTP_TIMEOUT         = 4
    CERTIFIER_OPT_HTTP_CONNECT_TIMEOUT = 5
    CERTIFIER_OPT_INPUT_P12_PATH       = 6
    CERTIFIER_OPT_INPUT_P12_PASSWORD   = 7
    CERTIFIER_OPT_OUTPUT_P12_PATH      = 8
    CERTIFIER_OPT_OUTPUT_P12_PASSWORD  = 9
    CERTIFIER_OPT_CA_INFO              = 10
    CERTIFIER_OPT_CA_PATH              = 11
    CERTIFIER_OPT_CRT                  = 12
    CERTIFIER_OPT_PROFILE_NAME         = 13
    CERTIFIER_OPT_OPTIONS      = 14
    CERTIFIER_OPT_ECC_CURVE_ID = 15
    CERTIFIER_OPT_SYSTEM_ID    = 16
    CERTIFIER_OPT_LOG_FILENAME = 17
    CERTIFIER_OPT_LOG_LEVEL    = 18
    CERTIFIER_OPT_AUTH_TOKEN  = 19
    CERTIFIER_OPT_OUTPUT_NODE = 20
    CERTIFIER_OPT_TARGET_NODE = 21
    CERTIFIER_OPT_ACTION      = 22
    CERTIFIER_OPT_INPUT_NODE  = 23
    CERTIFIER_OPT_DNS_SAN   = 24
    CERTIFIER_OPT_IP_SAN    = 25
    CERTIFIER_OPT_EMAIL_SAN = 26
    CERTIFIER_OPT_LOG_MAX_SIZE = 27
    CERTIFIER_OPT_TRACKING_ID = 28
    CERTIFIER_OPT_SOURCE        = 29
    CERTIFIER_OPT_CN_PREFIX     = 30
    CERTIFIER_OPT_VALIDITY_DAYS = 31
    CERTIFIER_OPT_EXT_KEY_USAGE = 32
    CERTIFIER_OPT_DOMAIN        = 33
    CERTIFIER__OPT_LOG_FUNCTION = 34
    CERTIFIER_OPT_CERT_MIN_TIME_LEFT_S = 35
    CERTIFIER_OPT_BOOL_FIRST = 36
    CERTIFIER_OPT_DEBUG_HTTP = CERTIFIER_OPT_BOOL_FIRST
    CERTIFIER_OPT_TRACE_HTTP = 37
    CERTIFIER_OPT_FORCE_REGISTRATION = 38
    CERTIFIER_OPT_MEASURE_PERFORMANCE = 39
    CERTIFIER_OPT_CERTIFICATE_LITE = 40
    CERTIFIER_OPT_MAC_ADDRESS = 41
    CERTIFIER_OPT_SIMULATION_CERT_EXP_DATE_BEFORE = 42
    CERTIFIER_OPT_SIMULATION_CERT_EXP_DATE_AFTER = 43
    CERTIFIER_OPT_AUTORENEW_INTERVAL = 44
    CERTIFIER_OPT_AUTORENEW_CERTS_PATH_LIST = 45
    CERTIFIER_OPT_USE_SCOPES = 46
    CERTIFIER_OPT_CERT_X509_OUT = 47
    CERTIFIER_OPT_MTLS_P12_PATH = 48
    CERTIFIER_OPT_MTLS_P12_PASSWORD = 49
    CERTIFIER_OPT_VERBOSE = 50

class CERTIFIER_OPT_OPTION(IntEnum):
    CERTIFIER_OPTION_DEBUG_HTTP          = 1
    CERTIFIER_OPTION_TRACE_HTTP          = 2
    CERTIFIER_OPTION_FORCE_REGISTRATION  = 4
    CERTIFIER_OPTION_MEASURE_PERFORMANCE = 8
    CERTIFIER_OPTION_CERTIFICATE_LITE    = 16
    CERTIFIER_OPTION_USE_SCOPES          = 32
    # 64, 128, 256, 512, 1024 are unused

class SECTIGO_OPT(IntEnum):
    SECTIGO_OPT_AUTH_TOKEN = 1
    SECTIGO_OPT_CFG_FILENAME = 2
    SECTIGO_OPT_VERBOSE = 3
    SECTIGO_OPT_COMMON_NAME = 4
    SECTIGO_OPT_SERIAL_NUMBER = 5
    SECTIGO_OPT_CERTIFICATE_ID = 6
    SECTIGO_OPT_REQUESTOR_EMAIL = 7
    SECTIGO_OPT_OWNER_LAST_NAME = 8
    SECTIGO_OPT_OWNER_FIRST_NAME = 9
    SECTIGO_OPT_OWNER_PHONE_NUMBER = 10
    SECTIGO_OPT_OWNER_EMAIL = 11
    SECTIGO_OPT_GROUP_NAME = 12
    SECTIGO_OPT_GROUP_EMAIL = 13
    SECTIGO_OPT_ID = 14
    SECTIGO_OPT_EMPLOYEE_TYPE = 15
    SECTIGO_OPT_SERVER_PLATFORM = 16
    SECTIGO_OPT_SENSITIVE = 17
    SECTIGO_OPT_PROJECT_NAME = 18
    SECTIGO_OPT_BUSINESS_JUSTIFICATION = 19
    SECTIGO_OPT_SUBJECT_ALT_NAMES = 20
    SECTIGO_OPT_IP_ADDRESSES = 21
    SECTIGO_OPT_CERTIFICATE_TYPE = 22
    SECTIGO_OPT_STATUS = 23
    SECTIGO_OPT_OFFSET = 24
    SECTIGO_OPT_LIMIT = 25
    SECTIGO_OPT_START_DATE = 26
    SECTIGO_OPT_END_DATE = 27
    SECTIGO_OPT_VALIDITY_START_DATE = 28
    SECTIGO_OPT_CERT_ORDER = 29
    SECTIGO_OPT_VALIDITY_END_DATE = 30
    SECTIGO_OPT_IS_CN_IN_SAN = 31
    SECTIGO_OPT_REQUEST_TYPE = 32 
    SECTIGO_OPT_TIMESTAMP = 33
    SECTIGO_OPT_REVOCATION_REQUEST_REASON = 34
    SECTIGO_OPT_LOG_LEVEL = 35
    SECTIGO_OPT_LOG_MAX_SIZE = 36
    SECTIGO_OPT_LOG_FILENAME = 37
    SECTIGO_OPT_OPTIONS = 38
    SECTIGO_OPT_CERT_OUTPUT = 39
    SECTIGO_OPT_SOURCE = 40
    SECTIGO_OPT_SECTIGO_URL = 41

ALLOWABLE_CHARACTERS = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnpqrstuvwxyz0123456879"

CERTIFIER_ERR_REGISTRATION_STATUS_X509_NONEXISTENT = (1 << 0)
CERTIFIER_ERR_REGISTRATION_STATUS_P12_NONEXISTENT = (1 << 1)
CERTIFIER_ERR_REGISTRATION_STATUS_CERT_EXPIRED_1 = (1 << 4)
CERTIFIER_ERR_REGISTRATION_STATUS_CERT_EXPIRED_2 = (1 << 5)
CERTIFIER_ERR_REGISTRATION_STATUS_CERT_ABOUT_TO_EXPIRE = (1 << 6)
CERTIFIER_ERR_REGISTRATION_STATUS_CERT_TIME_VALID = (1 << 7)

CERTIFIER_ERR_GET_CERT_STATUS_UNKNOWN = (1 << 8)
CERTIFIER_ERR_GET_CERT_STATUS_REVOKED = (1 << 9)
CERTIFIER_ERR_GET_CERT_STATUS_GOOD = (1 << 10)

PEM_BEGIN_CERTIFICATE = "-----BEGIN CERTIFICATE-----"
PEM_END_CERTIFICATE = "-----END CERTIFICATE-----"

SMALL_STRING_SIZE = 64
VERY_LARGE_STRING_SIZE = 2048

DEFAULT_LOG_LEVEL = 4
DEFAULT_LOG_MAX_SIZE = 5000000
DEFAULT_HTTP_TIMEOUT = 15
DEFAULT_HTTP_CONNECT_TIMEOUT = 15
DEFAULT_ECC_CURVE_ID = "prime256v1"
DEFAULT_OUTPUT_P12_PATH = "output.p12"
DEFAULT_AUTH_TYPE = "X509"
DEFAULT_CA_PATH = "/etc/ssl/certs"
DEFAULT_USER_CA_PATH = "/usr/local/etc/certfier"
DEFAULT_GLOBAL_CA_PATH = "/etc/certifier"
DEFAULT_CURDIR_CA_PATH = "."
DEFAULT_CERTIFER_URL = "https://certifier.xpki.io/v1/certifier"
DEFAULT_PROFILE_NAME = "XFN_Matter_OP_Class_3_ICA"
DEFAULT_CERT_MIN_TIME_LEFT_S = 90 * 24 * 60 * 60
DEFAULT_OPT_SOURCE = "unset-pycertifier-python"
DEFAULT_AUTORENEW_INTERVAL = 86400
DEFAULT_AUTORENEW_CERTS_PATH = "~/.pycertifier"

CERTIFIER_STATIC_URL =  "https://cert-static.xpki.io/v1/certifier"
DEFAULT_CERTIFER_URL = "https://certifier.xpki.io/v1/certifier"
IMPULSE_URL = "https://certs-dev.xpki.io/"

@contextmanager
def change_dir(destination):
    initial_dir = os.getcwd()

    try:
        os.chdir(destination)
        yield
    finally:
        os.chdir(initial_dir)


'''
Error Code Base Mapping
'''

get_cert_error = 1000
get_crt_token_error = 2000
get_cert_status_error = 3000
get_cert_validity_error = 4000
renew_cert_error = 5000
print_cert_error = 6000
revoke_error = 7000
property_error = 8000
certifier_error = 9000
